import java.util.Properties;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;


public class HQMessageGenerator
{
    private static final String JNDI_FACTORY_INITIAL = "org.jnp.interfaces.NamingContextFactory";
    private static final String JNDI_PROVIDER_URL = "jnp://localhost:5099";
    private static final String QUEUE_NAME = "queue/inbound_messages";

    public static void main(final String[] args)
        throws Exception
    {
        sendMessages("Initial", 3000, true);
        sendMessages("Failure", 1, false);
        sendMessages("Remaining", 3000, true);
    }

    private static void sendMessages(final String id, final int count, final boolean commit)
        throws Exception
    {
        Connection connection = null;
        Session session = null;
        InitialContext initialContext = null;
        Properties jndiProperties;

        System.out.println(id + ": Sending " + count + " msgs ...");

        try
        {
            jndiProperties = new Properties();
            jndiProperties.setProperty(Context.INITIAL_CONTEXT_FACTORY, JNDI_FACTORY_INITIAL);
            jndiProperties.setProperty(Context.PROVIDER_URL, JNDI_PROVIDER_URL);

            initialContext = new InitialContext(jndiProperties);

            Queue queue = (Queue) initialContext.lookup(QUEUE_NAME);

            ConnectionFactory cf = (ConnectionFactory) initialContext.lookup("ConnectionFactory");
            connection = cf.createConnection();
            session = connection.createSession(true, Session.SESSION_TRANSACTED);
            MessageProducer producer = session.createProducer(queue);

            StringBuilder messageBuilder = new StringBuilder();
            for (int i = 0; i < 32 * 1024; i++)
            {
                messageBuilder.append(" ");
            }

            for (int i = 0; i < count; i++)
            {
                final TextMessage tm = session.createTextMessage(messageBuilder.toString());
                producer.send(tm);
            }

            if (commit)
            {
                System.out.println(id + ": Committing ...");
                session.commit();
            }
            else
            {
                System.out.println(id + ": Skipping commit ...");
                // Rollback here doesn't help either ...
            }
        }
        finally
        {
            if (initialContext != null)
            {
                initialContext.close();
            }
            if (session != null)
            {
                session.close();
            }
            if (connection != null)
            {
                connection.close();
            }
        }
    }

}
